#!/bin/python
##############################################################
#    Yasuko Matsubara 
#    Date: 11-28-2014
##############################################################
import pylab
import numpy as np
import matplotlib.pyplot as plt
import math
from scipy.stats import norm
try:
    from sklearn.decomposition import FastICA, PCA
except:
    tool.error("can not find sklearn - please see http://scikit-learn.org")
import tool
nb=256
cF=8 # i.e., log2(nb)
BP=2 # bucets: (#BP, BP^2, BP^3, ...) 
DBG=0

ZERO=1.e-100
INF= 1.e+100
MAXBUCKETS=1.0/ZERO #1.0e+8 #ZERO


# Generate sample data
def genDemo():
    np.random.seed(0)
    n_samples = 2000
    time = np.linspace(0, 8, n_samples)
    # signals
    s1 = np.sin(2 * time)  
    s2 = np.sign(np.sin(3 * time))  
    s3 = np.sin(10 * time)
    S = np.c_[s1, s2, s3]
    S += 0.2 * np.random.normal(size=S.shape)  # Add noise
    S /= S.std(axis=0)  # Standardize data
    # Mix data
    #A = np.array([[1, 1, 1], [0.5, 2, 1.0], [1.5, 1.0, 2.0]])  # Mixing matrix
    A = np.array([[1, 1, 1], [0, 1, 1], [1, 0, 1], [0.5, 2, 1.0], [1.5, 1.0, 2.0]])  # Mixing matrix
    X = np.dot(S, A.T)  # Generate observations
    return X
    
# Compute ICA
def ica(X,k):
    try:
        ica = FastICA(n_components=k)
        S_ = ica.fit_transform(X)  # Reconstruct signals S_[Pp][k]
        A_ = ica.mixing_  # Get estimated mixing matrix  A_[d][k]
        M_ = ica.mean_    # Mean values
    except:
        tool.warning('ICA fitting error: return k=0 components ...')
        (m,n) = np.shape(X)
        S_=np.zeros((m,k)); A_=np.zeros((n,k)); M_=0 
    if(False):
        X_=np.dot(S_, A_.T) + M_ 
        print np.allclose(X, X_)
    return (S_, A_, M_)

def positive(S_,A_,M_):
    (n,k)=np.shape(A_)
    for i in range(0,k):
        maxA=A_[np.argmax(abs(A_[:,i])),i]
        #if(maxA<0):
        if(np.sum(A_[:,i])<0):
            S_[:,i]*=-1.0; A_[:,i]*=-1.0;
    return (S_, A_,M_)

def reconstruct(S_,A_,M_):
    X_=np.dot(S_, A_.T) + M_
    return X_

# Compute PCA
def pca(X,k):
    # For comparison, compute PCA
    pca = PCA(n_components=k)
    H = pca.fit_transform(X)  # Reconstruct signals based on orthogonal components
    return H

def bucketizeS(S_):
    (Pp,k)=np.shape(S_) # Pp: sequence length,  / #k: # of components
    S_b=S_.copy()
    # for each dimension
    for i in range(0,k):
        # find min, max values of S_
        maxV=max(S_[:,i]); minV=min(S_[:,i]); 
        bucketwd=(maxV-minV)/nb;
        S_b[:,i]=np.floor(S_[:,i]/bucketwd)*bucketwd
    return S_b    

def _coding(Xorg,Xrec):
    Delta = Xrec-Xorg;
    Delta[np.isnan(Delta)]=0
    if(DBG): print np.sqrt(np.mean(pow(Delta, 2))) # RMSE
    Delta=Delta*MAXBUCKETS # to avoid inf pdf
    std=np.std(Delta)+ZERO; mean=np.mean(Delta);
    pdfs=norm.pdf(Delta, mean, std)
    pdfs[pdfs>1]=1; 
    pdfs[pdfs<=ZERO]=ZERO
    cost = np.nansum(-np.log2(pdfs))
    return cost

def costT(X_org,S_,A_,M_):
    (Pp,k)=np.shape(S_) # Pp: sequence length,  / #k: # of components
    (d,k) =np.shape(A_) # d: # of data samples, / #k: # of components
    X_ = reconstruct(S_,A_,M_)
    CostC = _coding(X_,X_org) # diff, X_reconst vs. X_org
    CostM  = 0; 
    CostM += tool.log_s(k) + tool.log_s(d)
    CostM += cF*(Pp*k + d*k)  # S_[Pp][k], A_[d][k]
    CostT  = CostC + CostM
    if(math.isnan(CostT)): CostT=INF;
    if(DBG): print "k:%d, costM:%.0f, costC:%.0f, costT:%.0f"%(k, CostM, CostC, CostT)
    return CostT

# find optimal k with ICA
def ica_optk(X, mink, maxk):
    if(maxk<=1 or mink==maxk): 
        (S_,A_,M_)=ica(X,maxk); return (maxk,S_,A_,M_)
    # -------------------------------------------# 
    Cprev=INF; S_p=[];A_p=[];M_p=[]; optk=1;
    # try k = 1,2,...
    for k in range(1,maxk+1):
        #--- ICA - k components 
        (S_, A_, M_) = ica(X,k)
        S_=bucketizeS(S_)
        C=costT(X, S_,A_,M_)
        if(Cprev<C): break;
        Cprev=C; S_p=S_; A_p=A_; M_p=A_; optk=k;
    if(DBG): print 'optk:', optk
    # -------------------------------------------# 
    return (optk, S_p, A_p, M_p)
 
#---------------#
#     main      #
#---------------#
if __name__ == "__main__":

    #X = genDemo()
    X = pylab.loadtxt("./dat/E_games.txt")
    #X = pylab.loadtxt("./dat/E_socialmedia.txt")
    #X = pylab.loadtxt("./dat/E_s11121314.txt")
    #X = pylab.loadtxt("./dat/E_syns.txt")
    #X = pylab.loadtxt("./dat/E_retails")
    #X = pylab.loadtxt("./dat/season")
    #X = pylab.loadtxt("./dat/season_s")
    #X = X[:,[45-1, 221-1, 106-1, 30-1, 219-1]]
    #X = X[:,1:230]
    print np.shape(X)
    #X = pylab.loadtxt("./dat/Y")
    #X=X[:,0:60]
    #X = X[:,[0,1,10,11,20,21,30,31,39]]
    #-------------------------# 
    k=5 #len(X[0,:])-1
    (S_, A_, M_) = ica(X,k)
    #X_3 = reconstruct(S_,A_,M_)
    #-------------------------# 
    (optk, S_, A_, M_) = ica_optk(X,1,k)
    (S_, A_, M_) = positive(S_,A_,M_)
    #-------------------------# 
    H  = pca(X,k)
    #-------------------------# 

    # Plot results
    plt.figure()
    plt.subplot(1,2,1)
    plt.plot(X)
    plt.xlim(-4,len(X)+4)
    plt.subplot(1,2,2)
    plt.plot(S_)
    plt.xlim(-4,len(S_)+4)
    plt.subplots_adjust(0.09, 0.04, 0.94, 0.94, 0.26, 0.46)
    plt.show()
