#!/bin/python
##############################################################
#    Yasuko Matsubara 
#    Date: 11-10-2014
##############################################################
import numpy as np
import pylab
import sys
import argparse
import tool
import EcoWeb 
import time
import fit   
YES=1; NO=0;
WDSIZE=1
NORM=NO
INF=1.e+8
#------------------------#
#
#     main function   
#     e.g., python main_fit.py [-h] [-s SEQFN] [-t MTYPE] [-o OUTDIR] [-n N]
#
#------------------------#
if __name__ == "__main__":
    #--- arguments ---#
    parser = argparse.ArgumentParser()
    parser.add_argument("-s",  "--seqfn", type=str, help="input seqs filename")
    parser.add_argument("-t",  "--mtype", type=str, help="model type [u/k/p]")
    parser.add_argument("-o",  "--outdir",type=str, help="output dir (optional)")
    parser.add_argument("-n",  "--n",     type=int, help="set duration n (optional)")
    parser.add_argument("-d",  "--disp",  type=str, help="want GUI plots? - on (optional)")
    args = parser.parse_args()
    if(len(sys.argv)<5):
        parser.print_help()
    #--- check sequencefn, mtype ---#
    if(args.seqfn!=None and args.mtype!=None):
        seqfn = args.seqfn
        mtype = args.mtype
    else: parser.print_help()
    #--- check output dir ---#
    if(args.outdir!=None): outdir = args.outdir
    else: outdir = ''
    #--- check n (duration) ---#
    if(args.n>1): my_n=args.n
    else: my_n=INF

    #--- check wantDisplay ---#
    if(args.disp=='on'): wantDisp=YES; 
    else: wantDisp=NO;
    #--- print args ---#
    tool.comment('args')
    print "-------------------------"
    print "mtype: [%s]"%mtype
    print "seqfn: [%s]"%seqfn
    print "outdir:[%s]"%outdir
    print "-------------------------"

    #--- load data[d x n] ---#
    data=pylab.loadtxt(seqfn, ndmin=2).T
    (d,n)=np.shape(data)
    if(my_n<n):
        data=data[:,0:my_n]
        (d,n)=np.shape(data)  
    print "-------------------------"
    print 'd:' , d
    print 'n:' , n
    print "-------------------------"

    #--- smooth data (optional) ---#
    if(WDSIZE>1):
        data = tool.smoothMA(data, wd=WDSIZE)
        (d,n)=np.shape(data);    # data length
        data=data[:,0:n+1-WDSIZE]
        (d,n)=np.shape(data);    # data length
    #--- normalize data (optional) ---#
    if(NORM):
        data = tool.normalizeM(data, d, n)
    else:
        tool.warning('do not normalize')

    #--- create/load parameter set ---#
    model = EcoWeb.EcoWeb(d,n,data,mtype);
    #--- start fitting ---#
    if(mtype=='p'):
        model = fit.LMfit(model, outdir, 'proposed', wantDisp);
    else:
        model = fit.LMfit(model, outdir, 'naive', wantDisp);



