function [ output_args ] = M_plotModel( fn, outfn )
disp(['load data:', fn])
%set(gcf,'renderer','opengl');
figure(2); clf;

%% --- load base parameters --- %%
try
    fid=fopen([fn,'_model']);
    % read d, n
    d=fscanf(fid, 'd= %d\n');
    n=fscanf(fid, 'n= %d\n');
    % read X0
    fgets(fid); X0=fgets(fid); X0=str2num(X0);
    % read r
    fgets(fid); r=fgets(fid); r=str2num(r);
    % read K
    fgets(fid); K=fgets(fid); K=str2num(K);
    fgets(fid);
    for i=1:d
        A(i,:)=fgets(fid);
    end
    A=str2num(A);
    fclose(fid);
catch
    disp('no data'); return;
end
%% --- load base parameters --- %%
darks=2;
% full parameter set %
d, n, X0, r, K, A
A(A<0.05)=0; A(A>1)=1; % bound for viz 
%% plot base parameters %%
subplot(261); 
imagesc(-X0', [-2, 0]); axis image
largefont; title('P(0)')
set(gca, 'Xtick', []); set(gca, 'Ytick', 1:d); 
subplot(262); 
imagesc(-r', [-darks*max(r), 0]); axis image
largefont; title('r')
set(gca, 'Xtick', []); set(gca, 'Ytick', 1:d); 
subplot(263); 
imagesc(-K', [-3, 0]); axis image
largefont; title('K')
set(gca, 'Xtick', []); set(gca, 'Ytick', 1:d); 
subplot(222); 
imagesc(-A, [-darks*max(A(:)), 0]); axis image %colorbar()
largefont; title('A');
set(gca, 'Xtick', 1:d); set(gca, 'Ytick', 1:d); 
colormap(gray)
%% plot base parameters %%


%% --- load seasonal (exo) parameters --- %%
try
    E=load([fn,'_model_exo_E']);
    B=load([fn,'_model_exo_B']);
    W=load([fn,'_model_exo_W']);
    Pp=52;
    k=size(B,1);
catch
    disp('no exo data'); return;
end
%% --- load base parameters --- %%


%% plot seasonal (exo) parameters %%
subplot(245) 
imagesc(-W, [-darks*max(W(:)), 0]); largefont; axis image
set(gca, 'Xtick', 1:k); set(gca, 'Ytick', 1:d); 
title(num2str([d, k], 'W(%d x %d)'))
subplot(2,3,[5,6]) 
plotB(B, Pp,k)
%% plot seasonal (exo) parameters %%


%% save full parameter set plots
if(~isempty(outfn))
    saveas(gcf, [outfn, '_model.fig'])
    saveas(gcf, [outfn, '_model.png'])
    saveas(gcf, [outfn, '_model.eps'],'epsc')
end


if(1==1)
figure(3); clf;
%% plot only Bmatrix %%
plotB(B, Pp,k)
%% plot Umatrix %%
if(~isempty(outfn))
    saveas(gcf, [outfn, '_B.fig'])
    saveas(gcf, [outfn, '_B.eps'],'epsc')
end
end

[COLS, DCOLS]=M_getColors();
nodeColors = COLS*0.4+0.5;
w=0.8;
edgeColors = {'1', 'all', COLS(1,:)*w
              '2', 'all', COLS(2,:)*w
              '3', 'all', COLS(3,:)*w
              '4', 'all', COLS(4,:)*w
              '5', 'all', COLS(5,:)*w
              '6', 'all', COLS(6,:)*w};
%A=A';  % trans
A=A*20; % line width
A(A>7)=7; % UBound

mynames=strcat({''},int2str((1:d)'))';

edgeColors=edgeColors(1:d,:);
graphViz4Matlab_ex('-adjMat',A,'-nodeLabels',mynames,'-nodeColors',nodeColors,'-edgeColors', edgeColors);

if(~isempty(outfn))
    saveas(gcf, [outfn, '_chain.fig'])
    saveas(gcf, [outfn, '_chain.eps'],'epsc')
end


end

function [] = plotB(B, Pp,k)
BCOLS=[1 1 0; 1 0 1; 0 1 1; 1 0 0;  0 1 0; 0 0 1;]*0.3+0.2;
for i=1:k
    plot(B(i,:), 'Color', BCOLS(i,:)); hold on
end
hold off
xlim([1,Pp]); ylim([min(B(:)), max(B(:))]); largefont; 
xlabel('Time (weeks)')
ylabel('Value')
title(num2str([k], 'B (k=%d)'))
labels=strcat({'b'},int2str((1:k).')).';
legend(labels, 2) %, 'Orientation','horizontal')
M_setMonthLabels()
end